@@ -10,21 +10,25 @@ the terms of the GNU LGPL.
 '''
 
 if __name__ == '__main__':
-    import pytest
     import sys
+
+    import pytest
+
     sys.exit(pytest.main([__file__] + sys.argv[1:]))
 
-import subprocess
-import os
-import sys
-import pytest
-import stat
-import shutil
-import filecmp
 import errno
+import filecmp
+import os
+import shutil
+import stat
+import subprocess
+import sys
 from tempfile import NamedTemporaryFile
-from util import fuse_test_marker, wait_for_mount, umount, cleanup
+
+import pytest
+
 from pyfuse3 import _NANOS_PER_SEC
+from util import cleanup, fuse_test_marker, umount, wait_for_mount
 
 basename = os.path.join(os.path.dirname(__file__), '..')
 TEST_FILE = __file__
@@ -34,21 +38,20 @@ pytestmark = fuse_test_marker()
 with open(TEST_FILE, 'rb') as fh:
     TEST_DATA = fh.read()
 
+
 def name_generator(__ctr=[0]):
     __ctr[0] += 1
     return 'testfile_%d' % __ctr[0]
 
+
 @pytest.mark.parametrize('filename', ('hello.py', 'hello_asyncio.py'))
 def test_hello(tmpdir, filename):
     mnt_dir = str(tmpdir)
-    cmdline = [sys.executable,
-               os.path.join(basename, 'examples', filename),
-               mnt_dir ]
-    mount_process = subprocess.Popen(cmdline, stdin=subprocess.DEVNULL,
-                                     universal_newlines=True)
+    cmdline = [sys.executable, os.path.join(basename, 'examples', filename), mnt_dir]
+    mount_process = subprocess.Popen(cmdline, stdin=subprocess.DEVNULL, universal_newlines=True)
     try:
         wait_for_mount(mount_process, mnt_dir)
-        assert os.listdir(mnt_dir) == [ 'message' ]
+        assert os.listdir(mnt_dir) == ['message']
         filename = os.path.join(mnt_dir, 'message')
         with open(filename, 'r') as fh:
             assert fh.read() == 'hello world\n'
@@ -64,13 +67,11 @@ def test_hello(tmpdir, filename):
     else:
         umount(mount_process, mnt_dir)
 
+
 def test_tmpfs(tmpdir):
     mnt_dir = str(tmpdir)
-    cmdline = [sys.executable,
-               os.path.join(basename, 'examples', 'tmpfs.py'),
-               mnt_dir ]
-    mount_process = subprocess.Popen(cmdline, stdin=subprocess.DEVNULL,
-                                     universal_newlines=True)
+    cmdline = [sys.executable, os.path.join(basename, 'examples', 'tmpfs.py'), mnt_dir]
+    mount_process = subprocess.Popen(cmdline, stdin=subprocess.DEVNULL, universal_newlines=True)
     try:
         wait_for_mount(mount_process, mnt_dir)
         tst_write(mnt_dir)
@@ -94,14 +95,20 @@ def test_tmpfs(tmpdir):
     else:
         umount(mount_process, mnt_dir)
 
-def test_passthroughfs(tmpdir):
+
+@pytest.mark.parametrize('enable_writeback_cache', (True, False))
+def test_passthroughfs(tmpdir, enable_writeback_cache):
     mnt_dir = str(tmpdir.mkdir('mnt'))
     src_dir = str(tmpdir.mkdir('src'))
-    cmdline = [sys.executable,
-               os.path.join(basename, 'examples', 'passthroughfs.py'),
-               src_dir, mnt_dir ]
-    mount_process = subprocess.Popen(cmdline, stdin=subprocess.DEVNULL,
-                                     universal_newlines=True)
+    cmdline = [
+        sys.executable,
+        os.path.join(basename, 'examples', 'passthroughfs.py'),
+        src_dir,
+        mnt_dir,
+    ]
+    if enable_writeback_cache:
+        cmdline.append('--enable-writeback-cache')
+    mount_process = subprocess.Popen(cmdline, stdin=subprocess.DEVNULL, universal_newlines=True)
     try:
         wait_for_mount(mount_process, mnt_dir)
         tst_write(mnt_dir)
@@ -121,13 +128,14 @@ def test_passthroughfs(tmpdir):
         tst_truncate_path(mnt_dir)
         tst_truncate_fd(mnt_dir)
         tst_unlink(mnt_dir)
-        tst_passthrough(src_dir, mnt_dir)
+        tst_passthrough(src_dir, mnt_dir, enable_writeback_cache=enable_writeback_cache)
     except:
         cleanup(mount_process, mnt_dir)
         raise
     else:
         umount(mount_process, mnt_dir)
 
+
 def checked_unlink(filename, path, isdir=False):
     fullname = os.path.join(path, filename)
     if isdir:
@@ -139,17 +147,19 @@ def checked_unlink(filename, path, isdir=False):
     assert exc_info.value.errno == errno.ENOENT
     assert filename not in os.listdir(path)
 
+
 def tst_mkdir(mnt_dir):
     dirname = name_generator()
     fullname = mnt_dir + "/" + dirname
     os.mkdir(fullname)
     fstat = os.stat(fullname)
     assert stat.S_ISDIR(fstat.st_mode)
-    assert os.listdir(fullname) ==  []
-    assert fstat.st_nlink in (1,2)
+    assert os.listdir(fullname) == []
+    assert fstat.st_nlink in (1, 2)
     assert dirname in os.listdir(mnt_dir)
     checked_unlink(dirname, mnt_dir, isdir=True)
 
+
 def tst_symlink(mnt_dir):
     linkname = name_generator()
     fullname = mnt_dir + "/" + linkname
@@ -161,6 +171,7 @@ def tst_symlink(mnt_dir):
     assert linkname in os.listdir(mnt_dir)
     checked_unlink(linkname, mnt_dir)
 
+
 def tst_mknod(mnt_dir):
     filename = os.path.join(mnt_dir, name_generator())
     shutil.copyfile(TEST_FILE, filename)
@@ -171,6 +182,7 @@ def tst_mknod(mnt_dir):
     assert filecmp.cmp(TEST_FILE, filename, False)
     checked_unlink(filename, mnt_dir)
 
+
 def tst_chown(mnt_dir):
     filename = os.path.join(mnt_dir, name_generator())
     os.mkdir(filename)
@@ -192,6 +204,7 @@ def tst_chown(mnt_dir):
 
     checked_unlink(filename, mnt_dir, isdir=True)
 
+
 def tst_chmod(mnt_dir):
     filename = os.path.join(mnt_dir, name_generator())
     os.mkdir(filename)
@@ -206,12 +219,14 @@ def tst_chmod(mnt_dir):
 
     checked_unlink(filename, mnt_dir, isdir=True)
 
+
 def tst_write(mnt_dir):
     name = os.path.join(mnt_dir, name_generator())
     shutil.copyfile(TEST_FILE, name)
     assert filecmp.cmp(name, TEST_FILE, False)
     checked_unlink(name, mnt_dir)
 
+
 def tst_unlink(mnt_dir):
     name = os.path.join(mnt_dir, name_generator())
     data1 = b'foo'
@@ -222,11 +237,13 @@ def tst_unlink(mnt_dir):
         checked_unlink(name, mnt_dir)
         fh.write(data2)
         fh.seek(0)
-        assert fh.read() == data1+data2
+        assert fh.read() == data1 + data2
+
 
 def tst_statvfs(mnt_dir):
     os.statvfs(mnt_dir)
 
+
 def tst_link(mnt_dir):
     name1 = os.path.join(mnt_dir, name_generator())
     name2 = os.path.join(mnt_dir, name_generator())
@@ -247,6 +264,7 @@ def tst_link(mnt_dir):
     assert fstat1.st_nlink == 1
     os.unlink(name1)
 
+
 def tst_rename(mnt_dir):
     name1 = os.path.join(mnt_dir, name_generator())
     name2 = os.path.join(mnt_dir, name_generator())
@@ -266,6 +284,7 @@ def tst_rename(mnt_dir):
     assert os.path.basename(name2) in os.listdir(mnt_dir)
     os.unlink(name2)
 
+
 def tst_readdir(mnt_dir):
     dir_ = os.path.join(mnt_dir, name_generator())
     file_ = dir_ + "/" + name_generator()
@@ -279,7 +298,7 @@ def tst_readdir(mnt_dir):
 
     listdir_is = os.listdir(dir_)
     listdir_is.sort()
-    listdir_should = [ os.path.basename(file_), os.path.basename(subdir) ]
+    listdir_should = [os.path.basename(file_), os.path.basename(subdir)]
     listdir_should.sort()
     assert listdir_is == listdir_should
 
@@ -288,6 +307,7 @@ def tst_readdir(mnt_dir):
     os.rmdir(subdir)
     os.rmdir(dir_)
 
+
 def tst_truncate_path(mnt_dir):
     assert len(TEST_DATA) > 1024
 
@@ -310,10 +330,11 @@ def tst_truncate_path(mnt_dir):
     os.truncate(filename, size - 1024)
     assert os.stat(filename).st_size == size - 1024
     with open(filename, 'rb') as fh:
-        assert fh.read(size) == TEST_DATA[:size-1024]
+        assert fh.read(size) == TEST_DATA[: size - 1024]
 
     os.unlink(filename)
 
+
 def tst_truncate_fd(mnt_dir):
     assert len(TEST_DATA) > 1024
     with NamedTemporaryFile('w+b', 0, dir=mnt_dir) as fh:
@@ -334,7 +355,8 @@ def tst_truncate_fd(mnt_dir):
         os.ftruncate(fd, size - 1024)
         assert os.fstat(fd).st_size == size - 1024
         fh.seek(0)
-        assert fh.read(size) == TEST_DATA[:size-1024]
+        assert fh.read(size) == TEST_DATA[: size - 1024]
+
 
 def tst_utimens(mnt_dir, ns_tol=0):
     filename = os.path.join(mnt_dir, name_generator())
@@ -343,8 +365,8 @@ def tst_utimens(mnt_dir, ns_tol=0):
 
     atime = fstat.st_atime + 42.28
     mtime = fstat.st_mtime - 42.23
-    atime_ns = fstat.st_atime_ns + int(42.28*1e9)
-    mtime_ns = fstat.st_mtime_ns - int(42.23*1e9)
+    atime_ns = fstat.st_atime_ns + int(42.28 * 1e9)
+    mtime_ns = fstat.st_mtime_ns - int(42.23 * 1e9)
     os.utime(filename, None, ns=(atime_ns, mtime_ns))
 
     fstat = os.lstat(filename)
@@ -388,7 +410,8 @@ def tst_rounding(mnt_dir, ns_tol=0):
 
     checked_unlink(filename, mnt_dir, isdir=True)
 
-def tst_passthrough(src_dir, mnt_dir):
+
+def tst_passthrough(src_dir, mnt_dir, enable_writeback_cache: bool = False):
     # Test propagation from source to mirror
     name = name_generator()
     src_name = os.path.join(src_dir, name)
@@ -399,7 +422,7 @@ def tst_passthrough(src_dir, mnt_dir):
         fh.write('Hello, world')
     assert name in os.listdir(src_dir)
     assert name in os.listdir(mnt_dir)
-    assert_same_stats(src_name, mnt_name)
+    assert_same_stats(src_name, mnt_name, check_times=not enable_writeback_cache)
 
     # Test propagation from mirror to source
     name = name_generator()
@@ -411,7 +434,7 @@ def tst_passthrough(src_dir, mnt_dir):
         fh.write('Hello, world')
     assert name in os.listdir(src_dir)
     assert name in os.listdir(mnt_dir)
-    assert_same_stats(src_name, mnt_name)
+    assert_same_stats(src_name, mnt_name, check_times=not enable_writeback_cache)
 
     # Test propagation inside subdirectory
     name = name_generator()
@@ -426,16 +449,36 @@ def tst_passthrough(src_dir, mnt_dir):
         fh.write('Hello, world')
     assert name in os.listdir(src_dir)
     assert name in os.listdir(mnt_dir)
-    assert_same_stats(src_name, mnt_name)
+    assert_same_stats(src_name, mnt_name, check_times=not enable_writeback_cache)
 
-def assert_same_stats(name1, name2):
+
+def assert_same_stats(name1, name2, check_times: bool = True):
     stat1 = os.stat(name1)
     stat2 = os.stat(name2)
 
-    for name in ('st_atime_ns', 'st_mtime_ns', 'st_ctime_ns',
-                 'st_mode', 'st_ino', 'st_nlink', 'st_uid',
-                 'st_gid', 'st_size'):
+    for name in (
+        'st_atime_ns',
+        'st_mtime_ns',
+        'st_ctime_ns',
+        'st_mode',
+        'st_ino',
+        'st_nlink',
+        'st_uid',
+        'st_gid',
+        'st_size',
+    ):
         v1 = getattr(stat1, name)
         v2 = getattr(stat2, name)
-        assert  v1 == v2, 'Attribute {} differs by {} ({} vs {})'.format(
-            name, v1 - v2, v1, v2)
+
+        # Known bug, cf. https://github.com/libfuse/pyfuse3/issues/57
+        if name.endswith('_ns') and os.getenv('CI') == 'true':
+            continue
+
+        # When FUSE writeback cache is enabled, the kernel maintains mtime/ctime
+        # internally and only flushes them to the underlying filesystem on close.
+        # Until then, the timestamps reported for the passthrough mount and the
+        # backing directory may legitimately differ, so skip strict time checks.
+        if name.endswith('_ns') and not check_times:
+            continue
+
+        assert v1 == v2, 'Attribute {} differs by {} ({} vs {})'.format(name, v1 - v2, v1, v2)
