@@ -10,17 +10,19 @@ the terms of the GNU LGPL.
 '''
 
 if __name__ == '__main__':
-    import pytest
     import sys
+
+    import pytest
+
     sys.exit(pytest.main([__file__] + sys.argv[1:]))
 
-import pyfuse3
-from pyfuse3 import _NANOS_PER_SEC
+from pyfuse3 import _NANOS_PER_SEC, EntryAttributes
+
 
 def test_rounding():
     # Incorrect division previously resulted in rounding errors for
     # all dates.
-    entry = pyfuse3.EntryAttributes()
+    entry = EntryAttributes()
 
     # Approximately 67 years, ending in 999.
     # Note: 67 years were chosen to avoid y2038 issues (1970 + 67 = 2037).
@@ -34,10 +36,10 @@ def test_rounding():
     entry.st_ctime_ns = total
     entry.st_mtime_ns = total
     # Birthtime skipped -- only valid under BSD and OSX
-    #entry.st_birthtime_ns = total
+    # entry.st_birthtime_ns = total
 
     assert entry.st_atime_ns == total
     assert entry.st_ctime_ns == total
     assert entry.st_mtime_ns == total
     # Birthtime skipped -- only valid under BSD and OSX
-    #assert entry.st_birthtime_ns == total
+    # assert entry.st_birthtime_ns == total
